// npm install path-browserify --save
// vue-cli 使用 webpack 构建项目, webpack 自动帮我们处理了node内置模块，但是vite没有
// 因此, 只能使用一个第三方库替代nodejs内置库 path 的作用
// https://github.com/vitejs/vite/issues/2172
import path from 'path-browserify'
import type { RouteRecordNormalized, RouteRecordRaw } from 'vue-router'

/**
 * 获取所有的子集路由
 * @param routes 通过 router.getRoutes() 获取的完整路由表数组
 */
const getChildrenRoutes = (routes: RouteRecordNormalized[]) => {
  const childrenRoutes: RouteRecordRaw[] = []
  routes.forEach((route) => {
    if (route.children && route.children.length > 0) {
      childrenRoutes.push(...route.children)
    }
  })
  return childrenRoutes
}

/**
 * 处理脱离层级的路由
 */
export const filterRoutes = (routes: RouteRecordNormalized[]) => {
  // 1. 获取所有的子集路由 ( children 中的 )
  // tip: router.getRoutes() 返回的完整路由表数组中包含所有的 children 子集路由, 因此, 需要剔除位于 children 中的路由
  const childrenRoutes = getChildrenRoutes(routes)
  // 2. 根据子集路由进行查重操作
  return routes.filter(
    (route) =>
      !childrenRoutes.find((childRoute) => childRoute.path === route.path)
  )
}

/**
 * 根据 routes (filterRoutes 过滤后的) 数据, 返回对应的 menu 规则数据
 */
export const generateMenus = (routes: RouteRecordRaw[], basePath = '') => {
  const finalRoutes: RouteRecordRaw[] = []

  // 不满足该条件 `meta && meta.title && meta.icon` 的路由不应该存在
  routes.forEach((item) => {
    // 不存在 children && 不存在 meta 时, 直接 return
    if (isNull(item.meta) && isNull(item.children)) return
    // 存在 children, 不存在 meta, 迭代 generateMenus
    if (isNull(item.meta) && !isNull(item.children)) {
      finalRoutes.push(...generateMenus(item.children!))
      return
    }
    // 不存在 children, 存在 meta || 存在 children && 存在 meta
    // 因为最终的 menu 需要进行跳转, 此时我们需要合并 patch
    const routePath = path.resolve(basePath, item.path)
    // 路由分离之后, 可能存在 同名父路由 的情况
    let route = finalRoutes.find((route) => route.path === routePath)
    // 当前路由尚未加入到 finalRoutes
    if (!route) {
      route = {
        ...item,
        path: routePath,
        children: []
      }
      // icon && title: icon 与 title 全部存在
      if (route.meta?.title && route.meta.icon) {
        finalRoutes.push(route)
      }
    }

    // 存在 children && 存在 meta
    if (!isNull(item.children)) {
      route.children?.push(...generateMenus(item.children!, route.path))
    }
  })

  return finalRoutes
}

function isNull(data: any) {
  // data 为 null/undefined
  if (data == null) return true
  if (JSON.stringify(data) === '{}') return true
  if (JSON.stringify(data) === '[]') return true
  return false
}
